(function (global, factory) {
    typeof exports === 'object' && typeof module !== 'undefined' ? factory(exports, require('@angular/core'), require('rough-notation'), require('rxjs'), require('rxjs/operators'), require('@angular/common')) :
    typeof define === 'function' && define.amd ? define('ng-rough-notation', ['exports', '@angular/core', 'rough-notation', 'rxjs', 'rxjs/operators', '@angular/common'], factory) :
    (global = typeof globalThis !== 'undefined' ? globalThis : global || self, factory(global['ng-rough-notation'] = {}, global.ng.core, global.roughNotation, global.rxjs, global.rxjs.operators, global.ng.common));
}(this, (function (exports, core, roughNotation, rxjs, operators, common) { 'use strict';

    var defaultColors = {
        highlight: '#FFF176',
        circle: '#0D47A1',
        box: '#4A148C',
        'strike-through': '#1B5E1F',
        underline: '#B71C1B',
        'crossed-off': '#F57F17',
        bracket: '#FF0000',
    };
    var getDefaultTypeColor = function (type) {
        return defaultColors[type];
    };
    var defaultConfig = {
        type: 'highlight',
        color: getDefaultTypeColor('highlight'),
    };

    var RoughNotationService = /** @class */ (function () {
        function RoughNotationService(defaultConfig) {
            var _this = this;
            this.defaultConfig = defaultConfig;
            this.visibilitySubject = new rxjs.Subject();
            this.inputConfig = {};
            this.visibilityState$ = this.visibilitySubject.asObservable().pipe(operators.filter(function () { return !!_this.instance; }), operators.scan(function (currentState) { return !currentState; }, false), operators.tap(function (show) { return (show ? _this.instance.show() : _this.instance.hide()); }));
        }
        Object.defineProperty(RoughNotationService.prototype, "config", {
            /**
             * Generate config object from defaults and input overrides
             */
            get: function () {
                if (this.inputConfig.type && !this.inputConfig.color) {
                    // When a type is specified without a color
                    // Apply the default
                    this.inputConfig.color = getDefaultTypeColor(this.inputConfig.type);
                }
                return Object.assign(Object.assign({}, this.defaultConfig), this.inputConfig);
            },
            enumerable: false,
            configurable: true
        });
        RoughNotationService.prototype.instantiate = function (element) {
            this.instance = roughNotation.annotate(element, this.config);
            return this.instance;
        };
        RoughNotationService.prototype.destroy = function () {
            this.instance.remove();
        };
        RoughNotationService.prototype.toggleVisibility = function () {
            this.visibilitySubject.next();
        };
        return RoughNotationService;
    }());
    RoughNotationService.decorators = [
        { type: core.Injectable }
    ];
    RoughNotationService.ctorParameters = function () { return [
        { type: undefined, decorators: [{ type: core.Inject, args: ['DEFAULT_CONFIG',] }] }
    ]; };

    var RoughNotationDirective = /** @class */ (function () {
        function RoughNotationDirective(hostElement, renderer, service) {
            var _this = this;
            this.hostElement = hostElement;
            this.renderer = renderer;
            this.service = service;
            this.isShowingChange = new core.EventEmitter();
            // Storing initial host element color
            this.initialTextColor = hostElement.nativeElement.style.color;
            this.showStateSubscription = this.service.visibilityState$.subscribe(function (state) {
                _this.isShowingChange.emit(state);
                _this._show = state;
                if (_this.annotatedTextColor) {
                    // In case the user specified an annotation text color
                    // We set or reset it according to the state
                    var newColor = state ? _this.annotatedTextColor : _this.initialTextColor;
                    _this.renderer.setStyle(_this.hostElement.nativeElement, 'color', newColor);
                }
            });
        }
        Object.defineProperty(RoughNotationDirective.prototype, "roughNotation", {
            set: function (value) {
                if (typeof value === 'string') {
                    if (value) {
                        throw new Error('RoughAnnotationConfig object is expected as input');
                    }
                    this.service.inputConfig = {};
                }
                else {
                    this.service.inputConfig = value;
                }
            },
            enumerable: false,
            configurable: true
        });
        Object.defineProperty(RoughNotationDirective.prototype, "show", {
            get: function () {
                return this._show;
            },
            set: function (value) {
                if (value === false || this.instance) {
                    this.setVisibility(value);
                }
            },
            enumerable: false,
            configurable: true
        });
        Object.defineProperty(RoughNotationDirective.prototype, "instance", {
            get: function () {
                return this.service.instance;
            },
            enumerable: false,
            configurable: true
        });
        RoughNotationDirective.prototype.ngOnInit = function () {
            var _this = this;
            // Instantiate RoughNotation
            this.service.instantiate(this.hostElement.nativeElement);
            if (this._show !== false) {
                var animationDelay = this.service.config.animationDelay;
                if (animationDelay) {
                    // In case an animationDelay is specified
                    setTimeout(function () {
                        _this.setVisibility(true);
                    }, animationDelay);
                }
                else {
                    this.setVisibility(true);
                }
            }
        };
        RoughNotationDirective.prototype.setVisibility = function (show) {
            if (this._show !== show) {
                this.toggle();
                this._show = show;
            }
        };
        RoughNotationDirective.prototype.toggle = function () {
            this.service.toggleVisibility();
        };
        RoughNotationDirective.prototype.ngOnDestroy = function () {
            var _a;
            (_a = this.showStateSubscription) === null || _a === void 0 ? void 0 : _a.unsubscribe();
            this.service.destroy();
        };
        return RoughNotationDirective;
    }());
    RoughNotationDirective.decorators = [
        { type: core.Directive, args: [{
                    selector: '[roughNotation]',
                    providers: [RoughNotationService],
                    exportAs: 'roughNotation',
                },] }
    ];
    RoughNotationDirective.ctorParameters = function () { return [
        { type: core.ElementRef },
        { type: core.Renderer2 },
        { type: RoughNotationService }
    ]; };
    RoughNotationDirective.propDecorators = {
        roughNotation: [{ type: core.Input, args: ['roughNotation',] }],
        show: [{ type: core.Input }],
        instance: [{ type: core.Input, args: ['instance',] }],
        annotatedTextColor: [{ type: core.Input }],
        isShowingChange: [{ type: core.Output }]
    };

    var RoughNotationGroupComponent = /** @class */ (function () {
        function RoughNotationGroupComponent() {
            this._show = true;
        }
        Object.defineProperty(RoughNotationGroupComponent.prototype, "show", {
            set: function (value) {
                this.setVisibility(value);
            },
            enumerable: false,
            configurable: true
        });
        RoughNotationGroupComponent.prototype.ngAfterContentInit = function () {
            this.setVisibility(this._show);
        };
        RoughNotationGroupComponent.prototype.setVisibility = function (show) {
            if (this._show !== show) {
                this.toggleGroupMembers();
                this._show = show;
            }
        };
        RoughNotationGroupComponent.prototype.toggleGroupMembers = function () {
            this.annotations.forEach(function (annotation) { return annotation.toggle(); });
        };
        return RoughNotationGroupComponent;
    }());
    RoughNotationGroupComponent.decorators = [
        { type: core.Component, args: [{
                    selector: 'rough-notation-group',
                    template: "<ng-content></ng-content>\n",
                    styles: [""]
                },] }
    ];
    RoughNotationGroupComponent.propDecorators = {
        show: [{ type: core.Input }],
        annotations: [{ type: core.ContentChildren, args: [RoughNotationDirective, { descendants: true },] }]
    };

    var ɵ0 = defaultConfig;
    var RoughNotationModule = /** @class */ (function () {
        function RoughNotationModule() {
        }
        RoughNotationModule.forRoot = function (defaultConfigOverride) {
            return {
                ngModule: RoughNotationModule,
                providers: [
                    RoughNotationService,
                    {
                        provide: 'DEFAULT_CONFIG',
                        useValue: defaultConfigOverride !== null && defaultConfigOverride !== void 0 ? defaultConfigOverride : defaultConfig,
                    },
                ],
            };
        };
        return RoughNotationModule;
    }());
    RoughNotationModule.decorators = [
        { type: core.NgModule, args: [{
                    declarations: [RoughNotationDirective, RoughNotationGroupComponent],
                    imports: [common.CommonModule],
                    exports: [RoughNotationDirective, RoughNotationGroupComponent],
                    providers: [
                        RoughNotationService,
                        {
                            provide: 'DEFAULT_CONFIG',
                            useValue: ɵ0,
                        },
                    ],
                },] }
    ];

    /*
     * Public API Surface of ng-rough-notation
     */

    /**
     * Generated bundle index. Do not edit.
     */

    exports.RoughNotationDirective = RoughNotationDirective;
    exports.RoughNotationGroupComponent = RoughNotationGroupComponent;
    exports.RoughNotationModule = RoughNotationModule;
    exports.RoughNotationService = RoughNotationService;
    exports.ɵ0 = ɵ0;
    exports.ɵb = getDefaultTypeColor;
    exports.ɵc = defaultConfig;

    Object.defineProperty(exports, '__esModule', { value: true });

})));
//# sourceMappingURL=ng-rough-notation.umd.js.map
